# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


from __future__ import print_function
from __future__ import unicode_literals
import codecs


def bytestoint(data):
    return int(codecs.encode(data, 'hex_codec'), 16)



#!/usr/bin/env python
# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


from __future__ import print_function
from __future__ import unicode_literals
import unittest
from btctxstore import deserialize
from btctxstore import exceptions


class TestNulldataTxOut(unittest.TestCase):

    def test_max_data(self):

        # fourty bytes ok
        max_data = 40 * b"aa"
        deserialize.nulldata_txout(max_data)

        # > fourty bytes fails
        def callback():
            over_max_data = 41 * b"aa"
            deserialize.nulldata_txout(over_max_data)
        self.assertRaises(exceptions.MaxNulldataExceeded, callback)


class TestHash160DataTxOut(unittest.TestCase):

    def test_data_size(self):

        # twenty bytes ok
        max_data = 20 * b"aa"
        deserialize.hash160data_txout(max_data)

        # > twenty bytes fails
        def callback():
            over_max_data = 21 * b"aa"
            deserialize.hash160data_txout(over_max_data)
        self.assertRaises(exceptions.InvalidHash160DataSize, callback)

        # < twenty bytes fails
        def callback():
            over_max_data = 19 * b"aa"
            deserialize.hash160data_txout(over_max_data)
        self.assertRaises(exceptions.InvalidHash160DataSize, callback)


class TestKey(unittest.TestCase):

    def test_invalid_netcode(self):
        testnet_wif = "cNf7CMEHfD2jLtiTASbCAEneVnVmD4syA4A9KhUXqAkRs26Ke6xw"
        mainnet_wif = "KzU3561hXZwFPrzmHkJ6FLWvykFJMQnMEwSKXW5VPpz6HcxuvpZq"

        # test positive
        deserialize.key(True, testnet_wif)  # testnet
        deserialize.key(False, mainnet_wif)  # mainnet

        # testnet negative
        def callback():
            deserialize.key(False, testnet_wif)
        self.assertRaises(exceptions.InvalidWif, callback)

        # mainnet negative
        def callback():
            deserialize.key(True, mainnet_wif)
        self.assertRaises(exceptions.InvalidWif, callback)

        # testnet non string
        def callback():
            deserialize.key(False, None)
        self.assertRaises(exceptions.InvalidInput, callback)

        # mainnet non string
        def callback():
            deserialize.key(True, None)
        self.assertRaises(exceptions.InvalidInput, callback)

        # testnet garbage string
        def callback():
            deserialize.key(False, "garbage")
        self.assertRaises(exceptions.InvalidWif, callback)

        # mainnet garbage string
        def callback():
            deserialize.key(True, "garbage")
        self.assertRaises(exceptions.InvalidWif, callback)


class TestAddress(unittest.TestCase):

    def test_invalid_netcode(self):
        testnet_address = "mgBJ5bG9mQw8mHHcVEJghMamQEXeNLtvpt"
        mainnet_address = "1BTF7gU1EmgasGh85ypacDvsVKg4weZMfz"

        # testnet positive
        address = deserialize.address(True, testnet_address)
        self.assertEqual(testnet_address, address)

        # mainnet positive
        address = deserialize.address(False, mainnet_address)
        self.assertEqual(mainnet_address, address)

        # testnet negative
        def callback():
            deserialize.address(False, testnet_address)
        self.assertRaises(exceptions.InvalidAddress, callback)

        # mainnet negative
        def callback():
            deserialize.address(True, mainnet_address)
        self.assertRaises(exceptions.InvalidAddress, callback)

        # non string testnet
        def callback():
            deserialize.address(False, None)
        self.assertRaises(exceptions.InvalidInput, callback)

        # non string mainnet
        def callback():
            deserialize.address(True, None)
        self.assertRaises(exceptions.InvalidInput, callback)

        # garbage string testnet
        def callback():
            deserialize.address(False, "garbage")
        self.assertRaises(exceptions.InvalidAddress, callback)

        # garbage string mainnet
        def callback():
            deserialize.address(True, "garbage")
        self.assertRaises(exceptions.InvalidAddress, callback)



#!/usr/bin/env python
# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


from __future__ import print_function
from __future__ import unicode_literals
import unittest
from pycoin.key import validate
from btctxstore import BtcTxStore
from btctxstore import exceptions


S_WIF = "92JATRBTHRGAACcJb41dAGnh7kQ1wev27tcYWcGA2RZeUJLCcZo"
B_WIF = b"92JATRBTHRGAACcJb41dAGnh7kQ1wev27tcYWcGA2RZeUJLCcZo"
U_WIF = u"92JATRBTHRGAACcJb41dAGnh7kQ1wev27tcYWcGA2RZeUJLCcZo"
EXPECTED = "n3mW3o8XNMyH6xHWBkN98rm7zxxxswzpGM"


class TestGetAddress(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_standard(self):
        wif = self.api.create_key()
        address = self.api.get_address(wif)
        self.assertTrue(validate.is_address_valid(address, allowable_netcodes=['XTN']))

    def test_input_validation(self):

        # test correct types
        a = self.api.get_address(S_WIF)
        b = self.api.get_address(B_WIF)
        c = self.api.get_address(U_WIF)
        self.assertEqual(a, b, c)

        # TODO invalid types
        # TODO invalid input data

    def test_standards_compliant(self):
        wif = self.api.create_key()
        address = self.api.get_address(S_WIF)
        self.assertEqual(address, EXPECTED)
        # FIXME verify 3rd party produces this address


if __name__ == '__main__':
    unittest.main()



# source: http://eli.thegreenplace.net/2009/03/07/computing-modular-square-roots-in-python


def modular_sqrt(a, p):
    """ Find a quadratic residue (mod p) of 'a'. p
        must be an odd prime.

        Solve the congruence of the form:
            x^2 = a (mod p)
        And returns x. Note that p - x is also a root.

        0 is returned is no square root exists for
        these a and p.

        The Tonelli-Shanks algorithm is used (except
        for some simple cases in which the solution
        is known from an identity). This algorithm
        runs in polynomial time (unless the
        generalized Riemann hypothesis is false).
    """
    # Simple cases
    #
    if legendre_symbol(a, p) != 1:
        return 0
    elif a == 0:
        return 0
    elif p == 2:
        return p
    elif p % 4 == 3:
        return pow(a, (p + 1) // 4, p)

    # Partition p-1 to s * 2^e for an odd s (i.e.
    # reduce all the powers of 2 from p-1)
    #
    s = p - 1
    e = 0
    while s % 2 == 0:
        s //= 2
        e += 1

    # Find some 'n' with a legendre symbol n|p = -1.
    # Shouldn't take long.
    #
    n = 2
    while legendre_symbol(n, p) != -1:
        n += 1

    # Here be dragons!
    # Read the paper "Square roots from 1; 24, 51,
    # 10 to Dan Shanks" by Ezra Brown for more
    # information
    #

    # x is a guess of the square root that gets better
    # with each iteration.
    # b is the "fudge factor" - by how much we're off
    # with the guess. The invariant x^2 = ab (mod p)
    # is maintained throughout the loop.
    # g is used for successive powers of n to update
    # both a and b
    # r is the exponent - decreases with each update
    #
    x = pow(a, (s + 1) // 2, p)
    b = pow(a, s, p)
    g = pow(n, s, p)
    r = e

    while True:
        t = b
        m = 0
        for m in xrange(r):
            if t == 1:
                break
            t = pow(t, 2, p)

        if m == 0:
            return x

        gs = pow(g, 2 ** (r - m - 1), p)
        g = (gs * gs) % p
        x = (x * gs) % p
        b = (b * g) % p
        r = m


def legendre_symbol(a, p):
    """ Compute the Legendre symbol a|p using
        Euler's criterion. p is a prime, a is
        relatively prime to p (if p divides
        a, then a|p = 0)

        Returns 1 if a has a square root modulo
        p, -1 otherwise.
    """
    ls = pow(a, (p - 1) // 2, p)
    return -1 if ls == p - 1 else ls



#!/usr/bin/env python
# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


from __future__ import print_function
from __future__ import unicode_literals
import unittest
from pycoin.key import validate
from btctxstore import BtcTxStore
from btctxstore import exceptions


class TestCreateKey(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_standard(self):
        wif = self.api.create_key()
        self.assertTrue(validate.is_wif_valid(wif, allowable_netcodes=['XTN']))

    def test_random(self):
        a = self.api.create_key()
        b = self.api.create_key()
        self.assertTrue(a != b)

    def test_master_secret(self):
        a = self.api.create_key(master_secret="foo")
        b = self.api.create_key(master_secret="foo")
        self.assertEqual(a, b)

    def test_input_validation(self):

        # correct types
        a = self.api.create_key(master_secret="foo")
        b = self.api.create_key(master_secret=b"foo")
        c = self.api.create_key(master_secret=u"foo")
        self.assertEqual(a, b, c)
        self.assertTrue(self.api.create_key(master_secret="üöä") != None)
        self.assertTrue(self.api.create_key(master_secret=u"üöä") != None)

        # incorrect types
        def callback():
            self.api.create_key(master_secret=None)
        self.assertRaises(exceptions.InvalidInput, callback)
        def callback():
            self.api.create_key(master_secret=1)
        self.assertRaises(exceptions.InvalidInput, callback)
        def callback():
            self.api.create_key(master_secret=object())
        self.assertRaises(exceptions.InvalidInput, callback)

    def test_standards_compliant(self):
        pass # FIXME check generated against expected output from 3rd parties


if __name__ == '__main__':
    unittest.main()



# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


from __future__ import print_function
from __future__ import unicode_literals

import base64
from pycoin.serialize import b2h, b2h_rev


def data(data):
    return b2h(data)


def txid(txhash):
    return b2h_rev(txhash)


def txids(txhashs):
    return list(map(txid, txhashs))


def tx(tx):
    return tx.as_hex()


def signature(sigdata):
    return base64.b64encode(sigdata)


def utxos(spendables):
    def reformat(spendable):
        return {
            "txid": b2h_rev(spendable.tx_hash),
            "index": spendable.tx_out_index,
            "value": spendable.coin_value,
            "script": b2h(spendable.script)
        }
    return list(map(reformat, spendables))



# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


from __future__ import print_function
from __future__ import unicode_literals
from pycoin.encoding import from_long
from pycoin.encoding import to_long
from pycoin.encoding import byte_to_int


DUST_LIMIT = 548
MAX_NULLDATA = 40


def chunks(items, size):
    return [items[i:i+size] for i in range(0, len(items), size)]


def num_to_bytes(bytes_len, v):  # copied from pycoin.encoding.to_bytes_32
    v = from_long(v, 0, 256, lambda x: x)
    if len(v) > bytes_len:
        raise ValueError("input to num_to_bytes is too large")
    return ((b'\0' * bytes_len) + v)[-bytes_len:]


def num_from_bytes(bytes_len, v):  # copied from pycoin.encoding.to_bytes_32
    if len(v) != bytes_len:
        raise ValueError("input to num_from_bytes is wrong length")
    return to_long(256, byte_to_int, v)[0]



# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


from __future__ import print_function
from __future__ import unicode_literals
import binascii
from . import serialize
from . import deserialize
from . import control
from . import exceptions
from . import common
from . insight import InsightService  # XXX rm when added to pycoin


class BtcTxStore():  # TODO use apigen when ported to python 3
    """Bitcoin nulldata output io library."""

    def __init__(self, testnet=False, dryrun=False):
        self.testnet = deserialize.flag(testnet)
        self.dryrun = deserialize.flag(dryrun)
        if self.testnet:
            self.service = InsightService("https://test-insight.bitpay.com/")
        else:
            self.service = InsightService("https://insight.bitpay.com/")

    ###############################
    # wallets, keys and addresses #
    ###############################

    def create_wallet(self, master_secret=b""):
        """Create a BIP0032-style hierarchical wallet.

        @param: master_secret Create from master secret, otherwise random.
        """
        master_secret = deserialize.bytes_str(master_secret)
        bip32node = control.create_wallet(self.testnet,
                                          master_secret=master_secret)
        return bip32node.hwif(as_private=True)

    def get_key(self, hwif):  # TODO add optional path
        bip32node = deserialize.wallet(self.testnet, hwif)
        return bip32node.wif()

    def create_key(self, master_secret=b""):
        """Create new private key and return in wif format.
        
        @param: master_secret Create from master secret, otherwise random.
        """
        master_secret = deserialize.bytes_str(master_secret)
        bip32node = control.create_wallet(self.testnet,
                                          master_secret=master_secret)
        return bip32node.wif()

    def validate_address(self, address):  # TODO test
        try:
            deserialize.address(self.testnet, address)
            return True
        except exceptions.InvalidInput:
            return False

    def get_address(self, wif):
        """Return bitcoin address for given wallet. """
        return deserialize.key(self.testnet, wif).address()

    def validate_key(self, wif):  # TODO test
        try:
            deserialize.key(self.testnet, wif)
            return True
        except exceptions.InvalidInput:
            return False

    ###############
    # transaction #
    ###############

    def create_tx(self, txins=None, txouts=None, lock_time=0):
        """Create unsigned rawtx with given txins/txouts as json data.
        <txins>: '[{"txid" : hexdata, "index" : integer}, ...]'
        <txouts>: '[{"address" : hexdata, "value" : satoshis}, ...]'
        """
        txins = [] if txins is None else txins
        txouts = [] if txouts is None else txouts
        lock_time = deserialize.positive_integer(lock_time)
        txins = deserialize.txins(txins)
        txouts = deserialize.txouts(self.testnet, txouts)
        tx = control.create_tx(self.service, self.testnet, txins, txouts,
                               lock_time=lock_time)
        return serialize.tx(tx)

    def send(self, wifs, txouts, change_address=None, lock_time=0, fee=10000):
        """TODO add doc string"""
        # FIXME test!!
        rawtx = self.create_tx(txouts=txouts, lock_time=lock_time)
        rawtx = self.add_inputs(rawtx, wifs, change_address=change_address,
                                fee=fee)
        return self.publish(rawtx)

    def add_inputs(self, rawtx, wifs, change_address=None, fee=10000,
                   dont_sign=False):
        """Add sufficient inputs from given <wifs> to cover <rawtx> outputs
        and <fee>. If no <change_address> is given, change will be sent to
        first wif.
        """
        tx = deserialize.tx(rawtx)
        keys = deserialize.keys(self.testnet, wifs)
        fee = deserialize.positive_integer(fee)
        if change_address is not None:
            change_address = deserialize.address(self.testnet, change_address)
        tx = control.add_inputs(self.service, self.testnet, tx, keys,
                                change_address=change_address, fee=fee)

        if not dont_sign:
            tx = control.sign_tx(self.service, self.testnet, tx, keys)

        return serialize.tx(tx)

    def sign_tx(self, rawtx, wifs):
        """Sign <rawtx> with  given <wifs> as json data.
        <wifs>: '["privatekey_in_wif_format", ...]'
        """
        tx = deserialize.tx(rawtx)
        keys = deserialize.keys(self.testnet, wifs)
        tx = control.sign_tx(self.service, self.testnet, tx, keys)
        return serialize.tx(tx)

    #################
    # blockchain io #
    #################

    def retrieve_tx(self, txid):
        """Returns rawtx for <txid>."""
        txid = deserialize.txid(txid)
        tx = self.service.get_tx(txid)
        return serialize.tx(tx)

    def retrieve_utxos(self, addresses):
        """Get current utxos for <address>."""
        addresses = deserialize.addresses(self.testnet, addresses)
        spendables = control.retrieve_utxos(self.service, addresses)
        return serialize.utxos(spendables)

    def publish(self, rawtx):
        """Publish signed <rawtx> to bitcoin network."""
        tx = deserialize.signedtx(rawtx)
        if not self.dryrun:
            self.service.send_tx(tx)
        return serialize.txid(tx.hash())

    ###########
    # signing #
    ###########

    def sign_data(self, wif, hexdata):
        """Signing <hexdata> with <wif> private key."""
        data = deserialize.binary(hexdata)
        key = deserialize.key(self.testnet, wif)
        sigdata = control.sign_data(self.testnet, data, key)
        return serialize.signature(sigdata)

    def verify_signature(self, address, signature, hexdata):
        """Verify <signature> of <hexdata> by <address>."""
        try:
            address = deserialize.address(self.testnet, address)
            data = deserialize.binary(hexdata)
            signature = deserialize.signature(signature)
            return control.verify_signature(self.testnet, address,
                                            signature, data)
        except exceptions.InvalidAddress:
            return False

    def sign_unicode(self, wif, message):
        """Signing <unicode> with <wif> private key."""
        hexdata = binascii.hexlify(message.encode("utf-8"))
        return self.sign_data(wif, hexdata)

    def verify_signature_unicode(self, address, signature, message):
        """Verify <signature> of <unicode> by <address>."""
        hexdata = binascii.hexlify(message.encode("utf-8"))
        return self.verify_signature(address, signature, hexdata)

    ###############
    # hash160data #
    ###############

    def add_hash160data(self, rawtx, hexdata, dust_limit=common.DUST_LIMIT):
        """Writes <hexdata> as new Pay-to-PubkeyHash output to <rawtx>."""
        tx = deserialize.unsignedtx(rawtx)
        dust_limit = deserialize.positive_integer(dust_limit)
        hash160data_txout = deserialize.hash160data_txout(hexdata, dust_limit)
        tx = control.add_hash160data_output(tx, hash160data_txout)
        return serialize.tx(tx)

    def get_hash160data(self, rawtx, output_index):
        """TODO doc string"""
        tx = deserialize.unsignedtx(rawtx)
        output_index = deserialize.positive_integer(output_index)
        data = control.get_hash160_data(tx, output_index)
        return serialize.data(data)

    def store_hash160data(self, hexdata, wifs, change_address=None,
                          txouts=None, fee=10000, lock_time=0,
                          dust_limit=common.DUST_LIMIT):
        """TODO doc string"""
        rawtx = self.create_tx(txouts=txouts, lock_time=lock_time)
        rawtx = self.add_hash160data(rawtx, hexdata, dust_limit=dust_limit)
        rawtx = self.add_inputs(rawtx, wifs, change_address=change_address,
                                fee=fee)
        return self.publish(rawtx)

    def retrieve_hash160data(self, txid, output_index):
        """TODO doc string"""
        rawtx = self.retrieve_tx(txid)
        return self.get_hash160_data(rawtx, output_index)

    ############
    # nulldata #
    ############

    def add_nulldata(self, rawtx, hexdata):
        """Writes <hexdata> as new nulldata output to <rawtx>."""
        tx = deserialize.unsignedtx(rawtx)
        nulldata_txout = deserialize.nulldata_txout(hexdata)
        tx = control.add_nulldata_output(tx, nulldata_txout)
        return serialize.tx(tx)

    def get_nulldata(self, rawtx):
        """Returns nulldata from <rawtx> as hexdata."""
        tx = deserialize.tx(rawtx)
        index, data = control.get_nulldata(tx)
        return serialize.data(data)

    def store_nulldata(self, hexdata, wifs, change_address=None, txouts=None,
                       fee=10000, lock_time=0):
        """Store <hexdata> in blockchain and return new txid.
        Utxos taken from <wifs> and change sent to <change_address>.
        <wifs>: '["privatekey_in_wif_format", ...]'
        """
        rawtx = self.create_tx(txouts=txouts, lock_time=lock_time)
        rawtx = self.add_nulldata(rawtx, hexdata)
        rawtx = self.add_inputs(rawtx, wifs, change_address=change_address,
                                fee=fee)
        return self.publish(rawtx)

    def retrieve_nulldata(self, txid):
        """Returns nulldata stored in blockchain <txid> as hexdata."""
        rawtx = self.retrieve_tx(txid)
        return self.get_nulldata(rawtx)

    #############
    # data blob #
    #############

    def get_data_blob(self, rawtx):
        """TODO add docstring"""
        tx = deserialize.tx(rawtx)
        data = control.get_data_blob(tx)
        return serialize.data(data)

    def add_data_blob(self, rawtx, hexdata, dust_limit=common.DUST_LIMIT):
        """TODO add docstring"""
        tx = deserialize.tx(rawtx)
        data = deserialize.binary(hexdata)
        tx = control.add_data_blob(tx, data, dust_limit=dust_limit)
        return serialize.tx(tx)

    def store_data_blob(self, hexdata, wifs, change_address=None,
                        txouts=None, fee=10000, lock_time=0,
                        dust_limit=common.DUST_LIMIT):
        """TODO add docstring"""
        rawtx = self.create_tx(txouts=txouts, lock_time=lock_time)
        rawtx = self.add_data_blob(rawtx, hexdata, dust_limit=dust_limit)
        rawtx = self.add_inputs(rawtx, wifs, change_address=change_address,
                                fee=fee)
        return self.publish(rawtx)

    def retrieve_data_blob(self, txid):
        """TODO add docstring"""
        rawtx = self.retrieve_tx(txid)
        return self.get_data_blob(rawtx)

    #####################
    # broadcast message #
    #####################

    def add_broadcast_message(self, rawtx, message, sender_wif,
                              dust_limit=common.DUST_LIMIT):
        """TODO add docstring"""
        tx = deserialize.tx(rawtx)
        message = deserialize.unicode_str(message)
        sender_key = deserialize.key(self.testnet, sender_wif)
        tx = control.add_broadcast_message(self.testnet, tx, message,
                                           sender_key, dust_limit=dust_limit)
        return serialize.tx(tx)

    def get_broadcast_message(self, rawtx):
        """TODO add docstring"""
        tx = deserialize.tx(rawtx)
        result = control.get_broadcast_message(self.testnet, tx)
        result["signature"] = serialize.signature(result["signature"])
        return result

    def store_broadcast_message(self, message, sender_wif, wifs,
                                change_address=None, txouts=None, fee=10000,
                                lock_time=0, dust_limit=common.DUST_LIMIT):
        """TODO add docstring"""
        rawtx = self.create_tx(txouts=txouts, lock_time=lock_time)
        rawtx = self.add_broadcast_message(rawtx, message, sender_wif,
                                           dust_limit=dust_limit)
        rawtx = self.add_inputs(rawtx, wifs, change_address=change_address,
                                fee=fee)
        return self.publish(rawtx)

    def retrieve_broadcast_message(self, txid):
        """TODO add docstring"""
        rawtx = self.retrieve_tx(txid)
        return self.get_broadcast_message(rawtx)

    ########
    # misc #
    ########

    def split_utxos(self, wif, limit, fee=10000, max_outputs=100):
        """Split utxos of <wif> unitil <limit> or <max_outputs> reached."""
        key = deserialize.key(self.testnet, wif)
        limit = deserialize.positive_integer(limit)
        fee = deserialize.positive_integer(fee)
        max_outputs = deserialize.positive_integer(max_outputs)
        spendables = control.retrieve_utxos(self.service, [key.address()])
        txids = control.split_utxos(self.service, self.testnet, key,
                                    spendables, limit, fee=fee,
                                    max_outputs=max_outputs,
                                    publish=(not self.dryrun))
        return serialize.txids(txids)



#!/usr/bin/env python
# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


from __future__ import print_function
from __future__ import unicode_literals
import unittest
from pycoin.key import validate
from btctxstore import BtcTxStore
from btctxstore import exceptions


S_HWIF = "tprv8ZgxMBicQKsPeFZuyoMy197pX8iJeTasLVYEeb68cj489fmCz7kxP8vAfJp3rWebDT3nLvDsBaz3CqnUfdhr1wbjmhUNwK5Hf7dN2Er9btD"
B_HWIF = b"tprv8ZgxMBicQKsPeFZuyoMy197pX8iJeTasLVYEeb68cj489fmCz7kxP8vAfJp3rWebDT3nLvDsBaz3CqnUfdhr1wbjmhUNwK5Hf7dN2Er9btD"
U_HWIF = u"tprv8ZgxMBicQKsPeFZuyoMy197pX8iJeTasLVYEeb68cj489fmCz7kxP8vAfJp3rWebDT3nLvDsBaz3CqnUfdhr1wbjmhUNwK5Hf7dN2Er9btD"


class TestGetWalletKey(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_standard(self):
        hwif = self.api.create_wallet()
        wif = self.api.get_key(hwif)
        self.assertTrue(validate.is_wif_valid(wif, allowable_netcodes=['XTN']))

    def test_input_validation(self):
        
        # test correct types
        a = self.api.get_key(S_HWIF)
        b = self.api.get_key(B_HWIF)
        c = self.api.get_key(U_HWIF)
        self.assertEqual(a, b, c)

        # TODO invalid types
        # TODO invalid input data

    def test_standards_compliant(self):
        pass # FIXME check generated against expected output from 3rd parties


if __name__ == '__main__':
    unittest.main()



# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


from __future__ import print_function
from __future__ import unicode_literals


import base64
from pycoin.key import Key
from pycoin.tx.Tx import Tx
from pycoin.serialize import b2h, h2b, h2b_rev
from pycoin.tx.script import tools
from pycoin.encoding import bitcoin_address_to_hash160_sec
from pycoin.encoding import wif_to_secret_exponent
from pycoin.tx.TxOut import TxOut
from pycoin.tx.TxIn import TxIn
from pycoin.key import validate

from . import exceptions
from . import common


# TODO decorator to validate all io json serializable
# TODO use apigen when it supports python3


def unicode_str(s):
    if type(s) not in [type("string"), type(b"bytes"), type(u"unicode")]:
        raise exceptions.InvalidInput("Must be a string!")
    if isinstance(s, bytes):
        return s.decode('utf-8')
    return s


def bytes_str(s):
    if type(s) not in [type("string"), type(b"bytes"), type(u"unicode")]:
        raise exceptions.InvalidInput("Must be a string!")
    if not isinstance(s, bytes):
        return s.encode('utf-8')
    return s


def tx(rawtx):
    return Tx.tx_from_hex(rawtx)


def signedtx(rawtx):
    # FIXME validate tx is signed
    return Tx.tx_from_hex(rawtx)


def unsignedtx(rawtx):
    # FIXME validate tx is unsigned
    return Tx.tx_from_hex(rawtx)


def binary(hexdata):
    if isinstance(hexdata, bytes):
        hexdata = hexdata.decode("ascii")
    return h2b(hexdata)


def signature(sig):
    sig = base64.b64decode(sig)
    if len(sig) != 65:
        raise exceptions.InvalidInput("Signature must be 65 bytes long!")
    return sig


def integer(number):
    return int(number)


def flag(flag):
    return bool(flag)


def positive_integer(number):
    number = int(number)
    if number < 0:
        raise exceptions.InvalidInput("Integer may not be < 0!")
    return number


def txid(txhash):
    return h2b_rev(txhash)


def address(testnet, address):
    address = unicode_str(address)
    netcode = 'XTN' if testnet else 'BTC'
    if not validate.is_address_valid(address, allowable_netcodes=[netcode]):
        raise exceptions.InvalidAddress(address)
    return address


def addresses(testnet, addresses):
    return list(map(lambda addr: address(testnet, addr), addresses))


def txin(txhash, index):
    txhash = txid(txhash)
    index = positive_integer(index)
    return TxIn(txhash, index)


def txout(testnet, target_address, value):
    testnet = flag(testnet)
    target_address = address(testnet, target_address)
    value = positive_integer(value)
    prefix = b'\x6f' if testnet else b"\0"
    hash160 = b2h(bitcoin_address_to_hash160_sec(target_address, prefix))
    script_text = "OP_DUP OP_HASH160 %s OP_EQUALVERIFY OP_CHECKSIG" % hash160
    script_bin = tools.compile(script_text)
    return TxOut(value, script_bin)


def txins(data):
    return list(map(lambda x: txin(x['txid'], x['index']), data))


def txouts(testnet, data):
    return list(map(lambda x: txout(testnet, x['address'], x['value']), data))


def nulldata_txout(hexdata):
    data = binary(hexdata)
    if len(data) > common.MAX_NULLDATA:
        raise exceptions.MaxNulldataExceeded(len(data), common.MAX_NULLDATA)
    script_text = "OP_RETURN %s" % b2h(data)
    script_bin = tools.compile(script_text)
    return TxOut(0, script_bin)


def hash160data_txout(hexdata, dust_limit=common.DUST_LIMIT):
    data = binary(hexdata)
    if len(data) != 20:  # 160 bit
        raise exceptions.InvalidHash160DataSize(len(data))
    script_text = "OP_DUP OP_HASH160 %s OP_EQUALVERIFY OP_CHECKSIG" % b2h(data)
    script_bin = tools.compile(script_text)
    return TxOut(dust_limit, script_bin)


def secret_exponents(testnet, wifs):
    valid_prefixes = [b'\xef' if testnet else b'\x80']
    return list(map(lambda x: wif_to_secret_exponent(x, valid_prefixes), wifs))


def key(testnet, wif):
    wif = unicode_str(wif)
    netcode = 'XTN' if testnet else 'BTC'
    if not validate.is_wif_valid(wif, allowable_netcodes=[netcode]):
        raise exceptions.InvalidWif(wif)
    return Key.from_text(wif)


def keys(testnet, wifs):
    return list(map(lambda wif: key(testnet, wif), wifs))


def wallet(testnet, hwif):
    hwif = unicode_str(hwif)
    netcode = 'XTN' if testnet else 'BTC'
    if not validate.is_private_bip32_valid(hwif, allowable_netcodes=[netcode]):
        raise exceptions.InvalidHWIF(hwif)
    return Key.from_text(hwif)


def wallets(testnet, hwifs):
    return list(map(lambda hwif: wallet(testnet, hwif), hwifs))





# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


from __future__ import print_function
from __future__ import unicode_literals
import io
import re
import os
import six
import hashlib
import struct
import ecdsa
import math
import zlib
import binascii
from ecdsa.curves import SECP256k1
from pycoin.serialize.bitcoin_streamer import stream_bc_int
from pycoin.tx.Tx import Tx
from pycoin.encoding import hash160_sec_to_bitcoin_address
from pycoin.encoding import bitcoin_address_to_hash160_sec
from pycoin.encoding import public_pair_to_hash160_sec
from pycoin.encoding import double_sha256
from pycoin.tx.script import tools
from pycoin.serialize import h2b
from pycoin.tx.pay_to import build_hash160_lookup
from pycoin.tx import SIGHASH_ALL
from pycoin.tx.TxIn import TxIn
from pycoin.key.BIP32Node import BIP32Node
from . import util
from . import modsqrt
from . import deserialize
from . import serialize
from . import exceptions
from . import common


SIZE_PREFIX_BYTES = 2

# 6 byte data type keys to reduce chance of collision with random data
BROADCAST_MESSAGE_KEY_VERSON_01 = 'b220185f49e7'


def _address_to_hash160(testnet, address):
    prefix = b'\x6f' if testnet else b"\0"
    return bitcoin_address_to_hash160_sec(address, address_prefix=prefix)


def _hash160_to_address(testnet, hash160):
    prefix = b'\x6f' if testnet else b"\0"
    return hash160_sec_to_bitcoin_address(hash160, address_prefix=prefix)


def add_broadcast_message(testnet, tx, message, sender_key,
                          dust_limit=common.DUST_LIMIT):
    msg_data = message.encode('utf-8')
    signature = sign_data(testnet, msg_data, sender_key)
    hash160 = _address_to_hash160(testnet, sender_key.address())

    # compress after signing in case implementations compress differently
    msg_data = zlib.compress(msg_data, 9)

    data = binascii.unhexlify(BROADCAST_MESSAGE_KEY_VERSON_01)
    data += signature  # 65 byte message signature
    data += 7 * b'\0'  # 7 byte padding so sender hash160 aligns with txout
    data += hash160    # 20 byte aligned sender address (tx in history)
    data += msg_data   # the actual message data

    return add_data_blob(tx, data, dust_limit=dust_limit)


def get_broadcast_message(testnet, tx):

    try:
        data = get_data_blob(tx)
    except exceptions.NoDataBlob:
        raise exceptions.NoBroadcastMessage(tx)

    min_data = 6 + 65 + 7 + 20 + 0  # key + sig + padding + hash160 + message
    if len(data) < min_data:  # not enough data
        raise exceptions.NoBroadcastMessage(tx)

    # parse data
    msg_key = data[:6]  # get key
    data = data[6:]  # remove key
    signature = data[:65]  # get signature
    data = data[65:]  # remove signature
    data = data[7:]  # remove padding
    address = _hash160_to_address(testnet, data[:20])  # get address
    msg_data = data[20:]  # get message data

    if msg_key != binascii.unhexlify(BROADCAST_MESSAGE_KEY_VERSON_01):
        raise exceptions.NoBroadcastMessage(tx)

    # decompress before verification in case
    # implementations compress differently
    try:
        msg_data = zlib.decompress(msg_data)
    except zlib.error:
        raise exceptions.NoBroadcastMessage(tx)

    if not verify_signature(testnet, address, signature, msg_data):
        raise exceptions.NoBroadcastMessage(tx)  # invalid signature

    return {
        "address": address,
        "message": msg_data.decode('utf-8'),
        "signature": signature
    }


def get_data_blob(tx):
    # blob size and initial data stored in nulldata
    try:
        nulldata_index, nulldata = get_nulldata(tx)
    except exceptions.NoNulldataOutput:  # no nulldata output
        raise exceptions.NoDataBlob(tx)

    if len(nulldata) < SIZE_PREFIX_BYTES:  # no data size prefix
        raise exceptions.NoDataBlob(tx)

    # get size and initial data from nulldata
    size = common.num_from_bytes(SIZE_PREFIX_BYTES,
                                 nulldata[:SIZE_PREFIX_BYTES])
    data = nulldata[SIZE_PREFIX_BYTES:]  # strip size prefix

    if size < len(data):  # incorrect size prefix
        raise exceptions.NoDataBlob(tx)

    if size == len(data):  # nulldata was sufficient
        return data

    required_bytes = (size - len(data))
    required_hash160_outputs = int(math.ceil(required_bytes / 20.0))
    if (required_hash160_outputs + nulldata_index + 1) > len(tx.txs_out):
        raise exceptions.NoDataBlob(tx)  # not enough hash160 outputs for data

    for index in range(required_hash160_outputs):
        hash160_index = index + nulldata_index + 1
        data += get_hash160_data(tx, hash160_index)

    return data[:size]  # trim padding of last hash160output


def add_data_blob(tx, data, dust_limit=common.DUST_LIMIT):

    max_data_size = 2 ** (SIZE_PREFIX_BYTES * 8)
    if len(data) > max_data_size:
        raise exceptions.MaxDataBlobSizeExceeded(max_data_size, len(data))

    size_prefix = common.num_to_bytes(SIZE_PREFIX_BYTES, len(data))
    data = size_prefix + data

    # nulldata is sufficient
    if len(data) <= common.MAX_NULLDATA:
        nulldata_txout = deserialize.nulldata_txout(serialize.data(data))
        add_nulldata_output(tx, nulldata_txout)
        return tx

    # prefix and initial data stored in nulldata output
    nulldata = data[:common.MAX_NULLDATA]
    nulldata_txout = deserialize.nulldata_txout(serialize.data(nulldata))
    add_nulldata_output(tx, nulldata_txout)

    # remaining data stored in hash160data outputs
    for hash160data in common.chunks(data[common.MAX_NULLDATA:], 20):
        hexdata = serialize.data(hash160data)
        if len(hexdata) < 40:  # last entry needs padding
            hexdata = hexdata + '0' * (40 - len(hexdata))
        hash160data_txout = deserialize.hash160data_txout(hexdata, dust_limit)
        add_hash160data_output(tx, hash160data_txout)

    return tx


def _get_nulldata_output(tx):
    for index, out in enumerate(tx.txs_out):
        if re.match("^OP_RETURN", tools.disassemble(out.script)):
            return index, out
    return None, None


def add_nulldata_output(tx, nulldata_txout):
    index, out = _get_nulldata_output(tx)
    if out is not None:
        raise exceptions.ExistingNulldataOutput()
    # TODO validate transaction is unsigned
    tx.txs_out.append(nulldata_txout)
    # TODO validate transaction
    return tx


def add_hash160data_output(tx, hash160data_txout):
    # TODO validate transaction is unsigned
    tx.txs_out.append(hash160data_txout)
    # TODO validate transaction
    return tx


def get_hash160_data(tx, output_index):
    out = tx.txs_out[output_index]
    return h2b(tools.disassemble(out.script)[18:58])


def get_nulldata(tx):
    index, out = _get_nulldata_output(tx)
    if not out:
        raise exceptions.NoNulldataOutput(tx)
    data = h2b(tools.disassemble(out.script)[10:])
    return index, data


def create_tx(service, testnet, txins, txouts,
              lock_time=0, keys=None, publish=False):
    tx = Tx(1, txins, txouts, lock_time)
    if keys:
        tx = sign_tx(service, testnet, tx, keys)
    if publish:
        service.send_tx(tx)
    return tx


def sign_tx(service, testnet, tx, keys):
    netcode = 'XTN' if testnet else 'BTC'
    secretexponents = list(map(lambda key: key.secret_exponent(), keys))
    lookup = build_hash160_lookup(secretexponents)
    for txin_idx in range(len(tx.txs_in)):
        txin = tx.txs_in[txin_idx]
        utxo_tx = service.get_tx(txin.previous_hash)
        script = utxo_tx.txs_out[txin.previous_index].script
        tx.sign_tx_in(lookup, txin_idx, script, SIGHASH_ALL, netcode=netcode)
    return tx


def retrieve_utxos(service, addresses):
    spendables = service.spendables_for_addresses(addresses)
    spendables = sorted(spendables, key=lambda s: s.coin_value, reverse=True)
    return spendables


def find_txins(service, addresses, amount):
    spendables = retrieve_utxos(service, addresses)
    txins = []
    total = 0
    for spendable in spendables:
        total += spendable.coin_value
        txins.append(TxIn(spendable.tx_hash, spendable.tx_out_index))
        if total >= amount:
            return txins, total
    return txins, total


def _public_pair_to_address(testnet, public_pair, compressed):
    hash160 = public_pair_to_hash160_sec(public_pair, compressed=compressed)
    return _hash160_to_address(testnet, hash160)


def create_wallet(testnet, master_secret=b""):
    if not master_secret:
        master_secret = os.urandom(256)
    assert(isinstance(master_secret, bytes))
    netcode = 'XTN' if testnet else 'BTC'
    return BIP32Node.from_master_secret(master_secret, netcode=netcode)


def _encode_varint(value):
    f = io.BytesIO()
    stream_bc_int(f, value)
    return f.getvalue()


def _bitcoin_message_hash(data):
    prefix = b"\x18Bitcoin Signed Message:\n"
    varint = _encode_varint(len(data))
    return double_sha256(prefix + varint + data)


def _add_recovery_params(i, compressed, sigdata):
    params = 27  # signature parameters
    params += i  # add recovery parameter
    params += 4 if compressed else 0  # add compressed flag
    return struct.pack(">B", params) + sigdata


def sign_data(testnet, data, key):
    address = key.address()
    digest = _bitcoin_message_hash(data)
    secretexponent = key.secret_exponent()
    sigencode = ecdsa.util.sigencode_string

    # sign data
    pk = ecdsa.SigningKey.from_secret_exponent(secretexponent, curve=SECP256k1)
    sigdata = pk.sign_digest_deterministic(digest, hashfunc=hashlib.sha256,
                                           sigencode=sigencode)

    # add recovery params
    for i in range(4):
        for compressed in [True, False]:
            sig = _add_recovery_params(i, compressed, sigdata)
            if verify_signature(testnet, address, sig, data):
                return sig
    raise Exception("Failed to serialize signature!")


def _recover_public_key(G, order, r, s, i, e):
    """Recover a public key from a signature.
    See SEC 1: Elliptic Curve Cryptography, section 4.1.6, "Public
    Key Recovery Operation".
    http://www.secg.org/sec1-v2.pdf
    """
    c = ecdsa.ecdsa.curve_secp256k1

    # 1.1 Let x = r + jn
    x = r + (i // 2) * order

    # 1.3 point from x
    alpha = (x * x * x + c.a() * x + c.b()) % c.p()
    beta = modsqrt.modular_sqrt(alpha, c.p())
    y = beta if (beta - i) % 2 == 0 else c.p() - beta

    # 1.4 Check that nR is at infinity
    R = ecdsa.ellipticcurve.Point(c, x, y, order)

    rInv = ecdsa.numbertheory.inverse_mod(r, order)  # r^-1
    eNeg = -e % order  # -e

    # 1.6 compute Q = r^-1 (sR - eG)
    Q = rInv * (s * R + eNeg * G)
    return Q


def _parse_signature(sig, order):

    # parse r and s
    rsdata = sig[1:]
    r, s = ecdsa.util.sigdecode_string(rsdata, order)

    # parse parameters
    params = six.indexbytes(sig, 0) - 27
    if params != (params & 7):  # At most 3 bits
        raise exceptions.InvalidSignarureParameter()

    # get compressed parameter
    compressed = bool(params & 4)

    # get recovery parameter
    i = params & 3

    return rsdata, r, s, i, compressed


def verify_signature(testnet, address, sig, data):

    try:
        # parse sig data
        G = ecdsa.ecdsa.generator_secp256k1
        order = G.order()
        rsdata, r, s, i, compressed = _parse_signature(sig, order)
        digest = _bitcoin_message_hash(data)
        e = util.bytestoint(digest)

        # recover public key
        Q = _recover_public_key(G, order, r, s, i, e)
        pub = ecdsa.VerifyingKey.from_public_point(Q, curve=SECP256k1)

        # validate that recovered public key is correct
        sigdecode = ecdsa.util.sigdecode_string
        pub.verify_digest(rsdata, digest, sigdecode=sigdecode)

        # validate that recovered address is correct
        public_pair = [Q.x(), Q.y()]
        recoveredaddress = _public_pair_to_address(testnet, public_pair,
                                                   compressed)
        return address == recoveredaddress

    except AssertionError:  # _recover_public_key failed
        return False
    except exceptions.InvalidSignarureParameter:
        return False


def _take_txins(spendables, limit, max_outputs, fee):
    maxinput = limit * max_outputs + fee
    inputs = []
    while True:
        inputs_total = sum(list(map(lambda s: s.coin_value, inputs)))
        if inputs_total > maxinput or not spendables:
            break
        inputs.append(spendables.pop())
    txins = deserialize.txins(serialize.utxos(inputs))
    return txins, inputs_total


def _enough_to_split(spendables, fee, limit):
    total = sum(list(map(lambda s: s.coin_value, spendables)))
    return total >= (fee + limit * 2)


def _filter_dust(spendables, fee, limit):
    spendables = filter(lambda s: s.coin_value > fee, spendables)
    spendables = filter(lambda s: s.coin_value > limit, spendables)
    spendables = sorted(spendables, key=lambda s: s.coin_value)
    return spendables


def _outputs(testnet, inputs_total, fee, max_outputs, limit, key):
    txouts_total = inputs_total - fee
    if txouts_total > (max_outputs * limit):
        txouts_cnt = max_outputs
    else:
        txouts_cnt = txouts_total // limit
    txout_amount = txouts_total // txouts_cnt
    rounded_amount = (txouts_total - txout_amount * txouts_cnt)
    txouts = []
    for i in range(txouts_cnt):
        value = txout_amount + rounded_amount if i == 0 else txout_amount
        txouts.append(deserialize.txout(testnet, key.address(), value))
    assert(txouts_total == sum(list(map(lambda o: o.coin_value, txouts))))
    return txouts


def split_utxos(service, testnet, key, spendables, limit,
                fee=10000, max_outputs=100, publish=True):

    spendables = _filter_dust(spendables, fee, limit)
    if not _enough_to_split(spendables, fee, limit):
        return []
    txins, inputs_total = _take_txins(spendables, limit, max_outputs, fee)
    txouts = _outputs(testnet, inputs_total, fee, max_outputs, limit, key)
    tx = create_tx(service, testnet, txins, txouts,
                   keys=[key], publish=publish)

    # recurse for remaining spendables
    return [tx.hash()] + split_utxos(service, testnet, key, spendables, limit,
                                     fee=fee, max_outputs=max_outputs,
                                     publish=publish)


def add_inputs(service, testnet, tx, keys, change_address=None, fee=10000):

    # add inputs
    required = sum([out.coin_value for out in tx.txs_out]) + fee
    addresses = [key.address() for key in keys]
    txins, total = find_txins(service, addresses, required)
    if total < required:
        raise exceptions.InsufficientFunds(required, total)
    tx.txs_in += txins

    # add change output
    change_address = change_address if change_address else addresses[0]
    changeout = deserialize.txout(testnet, change_address, total - required)
    tx.txs_out.append(changeout)

    return tx



#!/usr/bin/env python
# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


from __future__ import print_function
from __future__ import unicode_literals
import json
import binascii
import unittest
from pycoin.key import validate
from btctxstore import BtcTxStore
from btctxstore import exceptions


fixtures = json.load(open("tests/fixtures.json"))


class TestAddInputs(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    # TODO test signing

    def test_add_inputs_withchange(self):
        txouts = fixtures["add_inputs"]["withchange"]["txouts"]
        wifs = fixtures["add_inputs"]["withchange"]["wifs"]
        change_address = fixtures["add_inputs"]["withchange"]["change_address"]
        expected = fixtures["add_inputs"]["withchange"]["expected"]
        rawtx = self.api.create_tx(txouts=txouts)
        result = self.api.add_inputs(rawtx, wifs, change_address,
                                     dont_sign=True)
        self.assertEqual(expected, result)

    def test_add_inputs_nochange(self):
        txouts = fixtures["add_inputs"]["nochange"]["txouts"]
        wifs = fixtures["add_inputs"]["nochange"]["wifs"]
        expected = fixtures["add_inputs"]["nochange"]["expected"]
        rawtx = self.api.create_tx(txouts=txouts)
        result = self.api.add_inputs(rawtx, wifs, dont_sign=True)
        self.assertEqual(expected, result)


class TestIO(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_readwrite_nulldata(self):
        rawtx = self.api.create_tx()
        rawtx = self.api.add_nulldata(rawtx, "f483")
        data = self.api.get_nulldata(rawtx)
        self.assertEqual(data, "f483")

    def test_readwrite_hash160data(self):
        rawtx = self.api.create_tx()
        rawtx = self.api.add_hash160data(rawtx, 10 * "f483")
        data = self.api.get_hash160data(rawtx, 0)
        self.assertEqual(data, 10 * "f483")

    def test_readwrite_data_blob(self):
        rawtx = self.api.create_tx()
        data_in = "f483" * 30 + "beef" * 30
        rawtx = self.api.add_data_blob(rawtx, data_in)
        data_out = self.api.get_data_blob(rawtx)
        self.assertEqual(data_in, data_out)

    def test_readwrite_broadcast_message(self):
        message = u"Ünicode test massage"
        sender_wif = fixtures["wallet"]["wif"]
        sender_address = fixtures["wallet"]["address"]
        sender_hash160hex = fixtures["wallet"]["hash160hex"]

        # create broadcast message tx
        rawtx = self.api.create_tx()
        rawtx = self.api.add_broadcast_message(rawtx, message, sender_wif)

        # check that sender address is aligned so its visible in history
        hash160hex = self.api.get_hash160data(rawtx, 3)
        self.assertEqual(hash160hex, sender_hash160hex)

        # get broadcast message from tx
        result = self.api.get_broadcast_message(rawtx)
        self.assertEqual(result["address"], sender_address)
        self.assertEqual(result["message"], message)
        
        # check signature is valid
        hex_message = binascii.hexlify(result["message"].encode('utf-8'))
        valid_signature = self.api.verify_signature(result["address"],
                                                    result["signature"],
                                                    hex_message)
        self.assertTrue(valid_signature)

    def test_only_one_nulldata_output(self):
        def callback():
            rawtx = self.api.create_tx()
            rawtx = self.api.add_nulldata(rawtx, "f483")
            self.api.add_nulldata(rawtx, "f483")  # writing second fails
        self.assertRaises(exceptions.ExistingNulldataOutput, callback)


class TestCreateTx(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_create_tx(self):
        lock_time = 0
        txins = fixtures["create_tx"]["txins"]
        txouts = fixtures["create_tx"]["txouts"]
        expected = fixtures["create_tx"]["expected"]
        result = self.api.create_tx(txins, txouts, lock_time)
        self.assertEqual(result, expected)


class TestRetrieveTx(unittest.TestCase):

    def test_retrieve_tx_testnet(self):
        api = BtcTxStore(dryrun=True, testnet=True)
        txid = fixtures["retrieve_tx"]["testnet"]["txid"]
        expected = fixtures["retrieve_tx"]["testnet"]["expected"]
        result = api.retrieve_tx(txid)
        self.assertEqual(result, expected)

    def test_retrieve_tx_mainnet(self):
        api = BtcTxStore(dryrun=True, testnet=False)
        txid = fixtures["retrieve_tx"]["mainnet"]["txid"]
        expected = fixtures["retrieve_tx"]["mainnet"]["expected"]
        result = api.retrieve_tx(txid)
        self.assertEqual(result, expected)


class TestGetUtxos(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_getutxos(self):
        address = fixtures["wallet"]["address"]
        expected = fixtures["getutxos"]["expected"]
        result = self.api.retrieve_utxos([address])
        self.assertEqual(result, expected)


class TestSignTx(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_sign_tx(self):
        txins = fixtures["sign_tx"]["txins"]
        txouts = fixtures["sign_tx"]["txouts"]
        wifs = fixtures["sign_tx"]["wifs"]
        expected = fixtures["sign_tx"]["expected"]
        rawtx = self.api.create_tx(txins, txouts)
        rawtx = self.api.add_nulldata(rawtx, "f483")
        result = self.api.sign_tx(rawtx, wifs)
        self.assertEqual(result, expected)


class TestStoreNulldata(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_store_nulldata_alpha(self):
        wifs = fixtures["store_nulldata"]["alpha"]["wifs"]
        result = self.api.store_nulldata("f483", wifs)
        expected = fixtures["store_nulldata"]["alpha"]["expected"]
        self.assertEqual(result, expected)

    def test_store_nulldata_beta(self):
        wifs = fixtures["store_nulldata"]["beta"]["wifs"]
        expected = fixtures["store_nulldata"]["beta"]["expected"]
        data = binascii.hexlify(b"github.com/F483/btctxstore")
        result = self.api.store_nulldata(data, wifs)
        self.assertEqual(result, expected)

    def test_store_nulldata_insufficient_funds(self):
        wifs = fixtures["store_nulldata"]["insufficient_funds"]["wifs"]
        data = binascii.hexlify(b"f483")

        def callback():
            self.api.store_nulldata(data, wifs)
        self.assertRaises(exceptions.InsufficientFunds, callback)

    def test_store_nulldata_txouts(self):
        wifs = fixtures["store_nulldata"]["txouts"]["wifs"]
        txouts = fixtures["store_nulldata"]["txouts"]["txouts"]
        expected = fixtures["store_nulldata"]["txouts"]["expected"]
        result = self.api.store_nulldata("f483", wifs, txouts=txouts)
        self.assertEqual(result, expected)

    def test_store_nulldata_change_address(self):
        _fixtures = fixtures["store_nulldata"]["change_address"]
        wifs = _fixtures["wifs"]
        change_address = _fixtures["change_address"]
        expected = _fixtures["expected"]
        result = self.api.store_nulldata("f483", wifs, change_address)
        self.assertEqual(result, expected)


class TestRetrieve(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_retrieve(self):
        txid = fixtures["retrieve"]["nulldata_txid"]
        result = self.api.retrieve_nulldata(txid)
        self.assertEqual(result, "f483")

    def test_retrieve_nothing(self):
        def callback():
            txid = fixtures["retrieve"]["nonulldata_txid"]
            result = self.api.retrieve_nulldata(txid)
        self.assertRaises(exceptions.NoNulldataOutput, callback)


class TestVerifySignature(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_verify_positive(self):
        _fixtures = fixtures["verify_signature"]["positive"]
        address = _fixtures["address"]
        signature = _fixtures["signature"]
        data = binascii.hexlify(b"testmessage")
        result = self.api.verify_signature(address, signature, data)
        self.assertEqual(result, True)

    def test_verify_incorrect_address(self):
        _fixtures = fixtures["verify_signature"]["incorrect_address"]
        address = _fixtures["address"]
        signature = _fixtures["signature"]
        data = binascii.hexlify(b"testmessage")
        result = self.api.verify_signature(address, signature, data)
        self.assertEqual(result, False)

    def test_verify_incorrect_signature(self):
        _fixtures = fixtures["verify_signature"]["incorrect_signature"]
        address = _fixtures["address"]
        signature = _fixtures["signature"]
        data = binascii.hexlify(b"testmessage")
        result = self.api.verify_signature(address, signature, data)
        self.assertEqual(result, False)

    def test_verify_incorrect_data(self):
        _fixtures = fixtures["verify_signature"]["incorrect_data"]
        address = _fixtures["address"]
        signature = _fixtures["signature"]
        data = binascii.hexlify(b"testmessagee")
        result = self.api.verify_signature(address, signature, data)
        self.assertEqual(result, False)

    def test_verify_signature_params(self):
        wif = "cSuT2J14dYbe1zvB5z5WTXeRcMbj4tnoKssAK1ZQbnX5HtHfW3bi"
        data = binascii.hexlify(b"testmessage")
        address = self.api.get_address(wif)
        sig = "///////////////////////////////////////////////////////////////////////////////////////="
        self.assertFalse(self.api.verify_signature(address, sig, data))


class TestSignData(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_sign_a(self):
        wif = fixtures["wallet"]["wif"]
        data = binascii.hexlify(b"testmessage")
        address = self.api.get_address(wif)
        sig = self.api.sign_data(wif, data)
        valid = self.api.verify_signature(address, sig, data)
        self.assertEqual(valid, True)

    def test_sign_b(self):
        wif = "cSuT2J14dYbe1zvB5z5WTXeRcMbj4tnoKssAK1ZQbnX5HtHfW3bi"
        data = binascii.hexlify(b"testmessage")
        address = self.api.get_address(wif)
        sig = self.api.sign_data(wif, data)
        valid = self.api.verify_signature(address, sig, data)
        self.assertEqual(valid, True)


class TestSignUnicode(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_sign_a(self):
        wif = fixtures["wallet"]["wif"]
        message = u"üöä"
        address = self.api.get_address(wif)
        sig = self.api.sign_unicode(wif, message)
        valid = self.api.verify_signature_unicode(address, sig, message)
        self.assertEqual(valid, True)

    def test_sign_b(self):
        wif = "cSuT2J14dYbe1zvB5z5WTXeRcMbj4tnoKssAK1ZQbnX5HtHfW3bi"
        message = u"üöä"
        address = self.api.get_address(wif)
        sig = self.api.sign_unicode(wif, message)
        valid = self.api.verify_signature_unicode(address, sig, message)
        self.assertEqual(valid, True)


class TestSplitUtxos(unittest.TestCase):

    def setUp(self):
        self.api = BtcTxStore(dryrun=True, testnet=True)

    def test_singleinput(self):
        wif = "cNHPbjVpkv4oqqKimBNp1UfQ2dhjETtRZw4KkHWtPgnU36SBtXub"
        # address n4RHA7mxH8EYV7wMS8evtYRYwCpQYz6KuE
        txids = self.api.split_utxos(wif, 10000000)  # 100mBTC
        self.assertEqual(len(txids), 1)

    def test_manyinputs(self):
        wif = "cRoboMG5KM19VP8ZcVCDXGCfi1JJraKpw58ofe8v57j7vqDxaQ5m"
        # address mqox6abLAiado9kFvX3EsHaVFbYVimSMCK
        txids = self.api.split_utxos(wif, 100000)  # 1mBTC
        self.assertEqual(len(txids), 6)


class TestValidateAddressTestnet(unittest.TestCase):

    def setUp(self):
        self.testnet_api = BtcTxStore(dryrun=True, testnet=True)
        self.mainnet_api = BtcTxStore(dryrun=True, testnet=False)

    def test_valid_string(self):
        address = 'migiScBNvVKYwEiCFhgBNGtZ87cdygtuSQ'
        self.assertTrue(self.testnet_api.validate_address(address))

    def test_valid_network(self):
        address = self.testnet_api.get_address(self.testnet_api.create_key())
        self.assertTrue(self.testnet_api.validate_address(address))

    def test_invalid_network(self):
        address = self.mainnet_api.get_address(self.mainnet_api.create_key())
        self.assertFalse(self.testnet_api.validate_address(address))

    def test_invalid_data(self):
        self.assertFalse(self.testnet_api.validate_address("f483"))

    def test_invalid_type(self):
        self.assertFalse(self.testnet_api.validate_address(None))


class TestValidateAddressMainnet(unittest.TestCase):

    def setUp(self):
        self.testnet_api = BtcTxStore(dryrun=True, testnet=True)
        self.mainnet_api = BtcTxStore(dryrun=True, testnet=False)

    def test_valid_string(self):
        address = '191GVvAaTRxLmz3rW3nU5jAV1rF186VxQc'
        self.assertTrue(self.mainnet_api.validate_address(address))

    def test_valid_network(self):
        address = self.mainnet_api.get_address(self.mainnet_api.create_key())
        self.assertTrue(self.mainnet_api.validate_address(address))

    def test_invalid_network(self):
        address = self.testnet_api.get_address(self.testnet_api.create_key())
        self.assertFalse(self.mainnet_api.validate_address(address))

    def test_invalid_data(self):
        self.assertFalse(self.mainnet_api.validate_address("f483"))

    def test_invalid_type(self):
        self.assertFalse(self.mainnet_api.validate_address(None))


class TestValidateKeyTestnet(unittest.TestCase):

    def setUp(self):
        self.testnet_api = BtcTxStore(dryrun=True, testnet=True)
        self.mainnet_api = BtcTxStore(dryrun=True, testnet=False)

    def test_valid_network(self):
        key = self.testnet_api.create_key()
        self.assertTrue(self.testnet_api.validate_key(key))

    def test_invalid_network(self):
        key = self.mainnet_api.create_key()
        self.assertFalse(self.testnet_api.validate_key(key))

    def test_invalid_data(self):
        self.assertFalse(self.testnet_api.validate_key("f483"))

    def test_invalid_type(self):
        self.assertFalse(self.testnet_api.validate_key(None))


class TestValidateKeyMainnet(unittest.TestCase):

    def setUp(self):
        self.testnet_api = BtcTxStore(dryrun=True, testnet=True)
        self.mainnet_api = BtcTxStore(dryrun=True, testnet=False)

    def test_valid_network(self):
        key = self.mainnet_api.create_key()
        self.assertTrue(self.mainnet_api.validate_key(key))

    def test_invalid_network(self):
        key = self.testnet_api.create_key()
        self.assertFalse(self.mainnet_api.validate_key(key))

    def test_invalid_data(self):
        self.assertFalse(self.mainnet_api.validate_key("f483"))

    def test_invalid_type(self):
        self.assertFalse(self.mainnet_api.validate_key(None))


if __name__ == '__main__':
    unittest.main()



# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


from __future__ import print_function
from __future__ import unicode_literals


class InvalidInput(Exception):  # FIXME ValueError instead?
    pass


class MaxNulldataExceeded(InvalidInput):

    def __init__(self, bytecount, max_nulldata):
        msg_txt = "Max nulldata exceeded: {0}bytes given, max {1}bytes!"
        msg = msg_txt.format(bytecount, max_nulldata)
        super(MaxNulldataExceeded, self).__init__(msg)


class InvalidHash160DataSize(InvalidInput):

    def __init__(self, bytecount):
        msg = "Hash160 data != 160bits: %sbits given!" % (bytecount * 8)
        super(InvalidHash160DataSize, self).__init__(msg)


class InvalidAddress(InvalidInput):

    def __init__(self, address):
        super(InvalidAddress, self).__init__("Invalid address '%s'!" % address)


class InvalidWif(InvalidInput):

    def __init__(self, wif):
        super(InvalidWif, self).__init__("Invalid wif '%s'!" % wif)


class InvalidHWIF(InvalidInput):

    def __init__(self, hwif):
        super(InvalidHWIF, self).__init__("Invalid hwif '%s'!" % hwif)


class InsufficientFunds(Exception):

    def __init__(self, required, available):
        msg = "Insufficient funds! Required: %s Available: %s"
        super(InsufficientFunds, self).__init__(msg % (required, available))


class ExistingNulldataOutput(Exception):

    def __init__(self):
        msg = "Transaction already has a nulldata output!"
        super(ExistingNulldataOutput, self).__init__(msg)


class InvalidSignarureParameter(Exception):

    def __init__(self):
        msg = "Invalid signature parameters!"
        super(InvalidSignarureParameter, self).__init__(msg)


class NoNulldataOutput(Exception):
    
    def __init__(self, tx):
        msg = "No nulldata output for tx '%s'!" % tx.as_hex()
        super(NoNulldataOutput, self).__init__(msg)


class NoDataBlob(Exception):
    
    def __init__(self, tx):
        msg = "No data blob stored in tx '%s'!" % tx.as_hex()
        super(NoDataBlob, self).__init__(msg)


class MaxDataBlobSizeExceeded(Exception):
    
    def __init__(self, max_data_size, data_size):
        msg_txt = "Max data size exceeded! '{0}' given, limit is '{1}' !"
        msg = msg_txt.format(max_data_size, data_size)
        super(MaxDataBlobSizeExceeded, self).__init__(msg)


class NoBroadcastMessage(Exception):
    
    def __init__(self, tx):
        msg = "No broadcast message stored in tx '%s'!" % tx.as_hex()
        super(NoBroadcastMessage, self).__init__(msg)



#!/usr/bin/env python
# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


import os
from setuptools import setup, find_packages


THISDIR = os.path.dirname(os.path.abspath(__file__))
os.chdir(THISDIR)


exec(open('btctxstore/version.py').read())  # load __version__
DOWNLOAD_BASEURL = "https://pypi.python.org/packages/source/a/btctxstore/"
DOWNLOAD_URL = DOWNLOAD_BASEURL + "btctxstore-%s.tar.gz" % __version__  # NOQA


setup(
    name='btctxstore',
    version=__version__,  # NOQA
    description=('Library to read/write data to bitcoin transaction outputs.'),
    long_description=open("README.rst").read(),
    keywords=("Bitcoin, OP_RETURN, store, tx, transaction"),
    url='https://github.com/F483/btctxstore/',
    author='Fabian Barkhau',
    author_email='fabian.barkhau@gmail.com',
    license='MIT',
    packages=find_packages(),
    download_url = DOWNLOAD_URL,
    test_suite="tests",
    install_requires=open("requirements.txt").readlines(),
    tests_require=[],  # use `pip install -r test_requirements.txt`
    zip_safe=False,
    classifiers=[
        # "Development Status :: 1 - Planning",
        # "Development Status :: 2 - Pre-Alpha",
        # "Development Status :: 3 - Alpha",
        "Development Status :: 4 - Beta",
        # "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



# coding: utf-8
# Copyright (c) 2015 Fabian Barkhau <fabian.barkhau@gmail.com>
# License: MIT (see LICENSE file)


# Store the version here so:
# 1) we don't load dependencies by storing it in __init__.py
# 2) we can import it in setup.py for the same reason
# 3) we can import it into your module module
__version__ = '4.4.0'
